# encoding: utf-8
# pylint: disable=missing-docstring,redefined-outer-name,no-self-use

from __future__ import unicode_literals

import os


DATA_PATH = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'data')
